package com.wtkj.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.wtkj.entity.Task;
import com.wtkj.entity.TaskFile;
import com.wtkj.entity.TaskLog;
import com.wtkj.service.ITaskFileService;
import com.wtkj.service.ITaskLogService;
import com.wtkj.service.ITaskService;
import com.wtkj.vo.TaskFileVO;
import com.wtkj.vo.TaskLogVO;
import com.wtkj.vo.TaskVO;
import com.wtkj.wrapper.TaskFileWrapper;
import com.wtkj.wrapper.TaskLogWrapper;
import com.wtkj.wrapper.TaskWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
 * @author Blizzard
 * @create at 2023-09-15 15:20
 * @describe
 */
@RestController
@AllArgsConstructor
@RequestMapping("/task")
@Api(value = "任务模块", tags = "任务模块")
public class TaskController {

	private final ITaskService taskService;

	private final ITaskLogService taskLogService;

	private final ITaskFileService taskFileService;

	@PostMapping("/submit")
	@ApiOperation(value = "创建、修改任务", notes = "")
	@ApiOperationSupport(order = 1)
	public R<Boolean> submitTask(@RequestBody @Valid Task task) {
		return R.status(taskService.submit(task));
	}

	/**
	 * 项目任务列表
	 */
	@GetMapping("/project-task-page")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "项目任务列表", notes = "传入projectId,分页参数")
	public R<IPage<TaskVO>> projectTask(@RequestParam Long projectId, Query query) {
		IPage<Task> page = taskService.projectTaskPage(projectId, Condition.getPage(query));
		return R.data(TaskWrapper.build().pageVO(page));
	}

	/**
	 * 任务详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "任务详情", notes = "传入任务主键id")
	public R<TaskVO> detail(@RequestParam Long id) {
		Task detail = taskService.getById(id);
		return R.data(TaskWrapper.build().entityVO(detail));
	}

	/**
	 * 任务成果文件
	 */
	@GetMapping("/result-file")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "任务成果文件", notes = "传入任务主键id")
	public R<List<TaskFileVO>> files(@RequestParam Long id) {
		List<TaskFile> files = taskFileService.fileList(id);
		return R.data(TaskFileWrapper.build().listVO(files));
	}

	/**
	 * 上传成果文件
	 */
	@GetMapping("/upload-file")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "上传成果文件", notes = "传入任务主键id，文件ids")
	public R saveFile(@RequestParam Long taskId, @RequestParam String ids) {
		return R.status(taskService.uploadFile(taskId, ids));
	}

	/**
	 * 删除附件
	 */
	@GetMapping("/delete-file")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "删除附件", notes = "传入主键ids")
	public R deleteFile(@RequestParam String ids) {
		return R.status(taskService.deleteFile(ids));
	}

	/**
	 * 移动文件
	 */
	@GetMapping("/move-file")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "移动文件", notes = "")
	public R moveFile(@RequestParam Long stageId, @RequestParam Long folderId, @RequestParam String fileIds) {
		return R.status(taskService.moveFile(stageId, folderId, fileIds));
	}

	/**
	 * 任务相关动态
	 */
	@GetMapping("/task-log")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "任务相关动态", notes = "传入任务主键id")
	public R<IPage<TaskLogVO>> taskLog(Long id, Query query) {
		LambdaQueryWrapper<TaskLog> lqw = new LambdaQueryWrapper<>();
		lqw.eq(TaskLog::getTaskId, id);
		lqw.orderByDesc(TaskLog::getCreateTime);
		IPage<TaskLog> page = taskLogService.page(Condition.getPage(query), lqw);
		return R.data(TaskLogWrapper.build().pageVO(page));
	}

	/**
	 *
	 */


}
