package com.wtkj.service.impl;

import com.wtkj.entity.*;
import com.wtkj.service.*;
import lombok.AllArgsConstructor;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.core.tool.utils.Func;
import org.springblade.system.entity.Dept;
import org.springblade.system.feign.ISysClient;
import org.springblade.system.user.entity.User;
import org.springblade.system.user.feign.IUserClient;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

import static com.wtkj.config.MagicValue.ZERO;

/**
 * @author Blizzard
 * @create at 2023-09-13 16:32
 * @describe
 */
@Service
@AllArgsConstructor
public class AsyncServiceImpl implements IAsyncService {

	private final ITypeAndStageService typeAndStageService;
	private final IProjectStageService stageService;
	private final IProjectAuthService projectAuthService;
	private final ISysClient sysClient;
	private final IUserClient userClient;
	private final ITaskLogService taskLogService;
	private final IProjectService projectService;
	private final IIndexMessageService indexMessageService;

	@Override
	@Async("asyncPoolTaskExecutor")
	public void createProject(Project project, BladeUser user) {
		//1.项目类型  --> 使用类型对应的模版阶段创建项目阶段
		Long projectType = project.getProjectType();
		List<TemplateTypeAndStage> templateStage = typeAndStageService.getStages(projectType);
		//2.创建项目阶段
		if (!CollectionUtils.isEmpty(templateStage)) {
			List<ProjectStage> stagesList = new ArrayList<>();
			templateStage.forEach(template -> {
				//项目的阶段记录表
				ProjectStage stage = new ProjectStage();
				stage.setProjectId(project.getId());
				stage.setName(template.getName());
				stage.setSort(template.getSort());
				stage.setCreateUser(project.getCreateUser());
				stage.setCreateDept(project.getCreateDept());
				stage.setCreateTime(DateUtil.now());
				stage.setUpdateTime(DateUtil.now());
				stage.setUpdateUser(project.getUpdateUser());
				stagesList.add(stage);
			});
			if (!CollectionUtils.isEmpty(stagesList)) {
				stageService.saveBatch(stagesList);
			}
		}

		//3.如果主管单位和当前登录用户不是同一机构
		String deptIdStr = user.getDeptId();
		if (deptIdStr != null) {
			Long deptId = Func.firstLong(deptIdStr);
			if (!deptId.equals(project.getCompetentUnit())) {
				ProjectAuth auth = new ProjectAuth();
				auth.setUserId(user.getUserId());
				R<Dept> rpc = sysClient.getDept(deptId);
				if (rpc.isSuccess()) {
					String ancestors = rpc.getData().getAncestors();
					if (ancestors != null && ancestors.contains(String.valueOf(ZERO))) {
						List<Long> longList = Func.toLongList(ancestors);
						if (!CollectionUtils.isEmpty(longList)) {
							if (longList.size() > 2) {
								auth.setTopDept(longList.get(1));
							} else {
								auth.setTopDept(Long.valueOf(deptIdStr));
							}
						}
					}
				}
				auth.setUserDept(deptId);
				auth.setProjectId(project.getId());
				projectAuthService.save(auth);
			}
		}

	}

	@Async("asyncPoolTaskExecutor")
	@Override
	public void removeProject(List<Long> longList) {

	}

	@Override
	public void createTaskLog(Task task) {
		//1.任务日志
		Long createUser = task.getCreateUser();
		if (createUser != null) {
			R<User> userR = userClient.userInfoById(createUser);
			if (userR.isSuccess()) {
				List<TaskLog> logs = new ArrayList<>();
				TaskLog log = new TaskLog();
				log.setTaskId(task.getId());
				log.setContent(userR.getData().getName() + " 创建了任务");
				log.setCreateDept(task.getCreateDept());
				log.setCreateUser(task.getCreateUser());
				log.setUpdateUser(task.getUpdateUser());
				log.setCreateTime(DateUtil.now());
				log.setUpdateTime(DateUtil.now());
				logs.add(log);
				if (task.getOrgDeptId() != null) {
					TaskLog taskLog = new TaskLog();
					taskLog.setTaskId(task.getId());
					taskLog.setContent(userR.getData().getName() + " 指定了执行单位");
					taskLog.setCreateDept(task.getCreateDept());
					taskLog.setCreateUser(task.getCreateUser());
					taskLog.setUpdateUser(task.getUpdateUser());
					taskLog.setCreateTime(DateUtil.now());
					taskLog.setUpdateTime(DateUtil.now());
					logs.add(log);
				}
				taskLogService.saveBatch(logs);
			}
		}
		// todo 2.任务下发通知
	}

	@Override
	public void submitTaskOrgDept(Long userId, Task task) {
		R<User> userR = userClient.userInfoById(userId);
		if (userR.isSuccess()) {
			TaskLog log = new TaskLog();
			log.setTaskId(task.getId());
			log.setContent(userR.getData().getName() + " 指定了执行单位");
			log.setCreateDept(task.getCreateDept());
			log.setCreateUser(task.getCreateUser());
			log.setUpdateUser(task.getUpdateUser());
			log.setCreateTime(DateUtil.now());
			log.setUpdateTime(DateUtil.now());
			taskLogService.save(log);
		}
	}

	@Override
	public void submitTaskMessage(Long userId, Task task) {
		Long createUser = task.getCreateUser();
		Long projectId = task.getProjectId();
		String title = task.getTitle();
		R<User> userR = userClient.userInfoById(userId);
		Project project = projectService.getById(projectId);
		if (userR.isSuccess() && project != null) {
			IndexMessage message = new IndexMessage();
			message.setCategory(1);
			//发送给任务创建者
			message.setToUser(createUser);
			message.setContent(userR.getData().getName() + "上传了" + project.getName() + title + "任务的文件，请查阅");
			message.setOpenUrl("/task");
			message.setCreateDept(task.getCreateDept());
			message.setCreateUser(userId);
			message.setUpdateUser(userId);
			message.setCreateTime(DateUtil.now());
			message.setUpdateTime(DateUtil.now());
			indexMessageService.save(message);
		}
	}


}
